# /u/sy/beebe/tex/tgrind/tgrind-3.00/build-summary.awk, Tue Jun 16 07:49:22 1998
# Edit by Nelson H. F. Beebe <beebe@math.utah.edu>
# ========================================================================
# Read a typescript from the build-all.sh script and produce a list
# of compiler successes as a sorted HTML table:
#
# Usage:
#	gawk -f build-summary.awk script-file
#
# [09-Jul-1998] -- Update to omit flex/lex in title if only one is present.
#		   Extract compiler from "configuring for..." line instead
#		   of from "/xxx/yyy/..." lines, so that we can preserve
#		   special compiler options.
#		   Convert spaces in compiler name to underscore; the
#		   companion build-summary.sh script converts them back
#		   to "&nbsp;" after prettyprinting, because html-pretty
#		   currently will break at "&nbsp;" 
# [16-Jun-1998] -- adapt for use with tgrind-3.00
# [01-Jun-1998]
# ========================================================================

BEGIN		{ LexCount = 0; FlexCount = 0; Lexer = "flex" }

/^configuring for/	{
				Compiler = $0;
				sub("^configuring for *","",Compiler)
				sub(" *with configure options:.*$","",Compiler)
				gsub(" +","_",Compiler)
			}


/^\/[^d][^e][^v]/ { Diff_Failure = 0; next }

/^MACHINETYPE:/	{ 
		    MachineType = $0
		    sub(/^MACHINETYPE: */,"",MachineType)
		}

/^f?lex /	{ Lexer = $1 }

/^[<>]/		{ Diff_failure = 1; next }

# /^diff /	{ if (!Diff_Failure) add_result(); Diff_Failure = 0; next }

# /^check020/	{ if (!Diff_Failure) add_result(); Diff_Failure = 0; next }

/^There is no validation suite for dired...you must run it manually/ { add_result(); next }
/^There is no/ { add_result(); next }

END		{ print_results() }

#=======================================================================

function add_result()
{
    getline x
    if (x ~ /[A-Z][a-z][a-z] +[A-Z][a-z][a-z] +[0-9]+ +[0-9]+:[0-9]+:[0-9]+ +[A-Z]* *[12][0-9][0-9][0-9]/)
    {
	if ((Lexer == "lex") && (!(triple() in Triple)))
	{
	    if (MachineType in Lex)
		Lex[MachineType] = Lex[MachineType] ",\t" Compiler
	    else
		Lex[MachineType] = html_table_row(MachineType, Compiler)
	    LexCount++
	    Triple[triple()] = 1
	}
	else if ((Lexer == "flex") && (!(triple() in Triple)))
	{
	    if (MachineType in Flex)
		Flex[MachineType] = Flex[MachineType] ",\t" Compiler
	    else
		Flex[MachineType] = html_table_row(MachineType, Compiler)
	    FlexCount++
	    Triple[triple()] = 1
	}
	### print "DEBUG", Lexer, LexCount, FlexCount, $0
    }
}


function html_table_row(machine,compiler)
{
    gsub(/; */,"</TD>\t<TD>",machine)
    return (sprintf("<TR>\t<TD>%-48s</TD>\t<TD>\t%s",machine,compiler))
}


function print_results()
{
    if (LexCount > 0)
	print_table((FlexCount > 0) ? "lex" : "",Lex,LexCount)
    if (FlexCount > 0)
	print_table((LexCount > 0) ? "flex" : "",Flex,FlexCount)
}


function print_table(title,table,count, name,sortpipe)
{
    if (title == "")
	print "<H3>" count " compilers</H3>"
    else
	print "<H3><TT>" title "</TT> (" count " compilers)</H3>"
	
    print "<TABLE>" 
    print "<TR><TD><B>Machine and model</B></TD><TD><B>O/S</B></TD><TD><B>Compilers</B></TD></TR>"
    sortpipe =  "sort | tr '\t' '\n'"
    for (name in table)
	print table[name] "\t</TD>\t</TR>" | sortpipe
    close (sortpipe)
    print "</TABLE>"
}


function triple()
{
    return (MachineType SUBSEP Lexer SUBSEP Compiler)  
}
